<?php

namespace Oro\Bundle\SegmentBundle\Twig;

use Oro\Bundle\SegmentBundle\Event\ConditionBuilderOptionsLoadEvent;
use Oro\Bundle\SegmentBundle\Event\WidgetOptionsLoadEvent;
use Psr\Container\ContainerInterface;
use Symfony\Component\DependencyInjection\ServiceSubscriberInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Twig\Extension\AbstractExtension;
use Twig\TwigFunction;

/**
 * Provides Twig functions to retrieve segment query builder configuration:
 *   - update_segment_widget_options
 *   - update_segment_condition_builder_options
 */
class SegmentExtension extends AbstractExtension implements ServiceSubscriberInterface
{
    const NAME = 'oro_segment';

    /** @var ContainerInterface */
    private $container;

    /**
     * @param ContainerInterface $container
     */
    public function __construct(ContainerInterface $container)
    {
        $this->container = $container;
    }

    /**
     * {@inheritdoc}
     */
    public function getFunctions()
    {
        return [
            new TwigFunction('update_segment_widget_options', [$this, 'updateSegmentWidgetOptions']),
            new TwigFunction(
                'update_segment_condition_builder_options',
                [$this, 'updateSegmentConditionBuilderOptions']
            ),
        ];
    }

    /**
     * @param array $widgetOptions
     * @param string|null $type
     *
     * @return array
     */
    public function updateSegmentWidgetOptions(array $widgetOptions, $type = null)
    {
        $eventDispatcher = $this->container->get('event_dispatcher');

        if (!$eventDispatcher->hasListeners(WidgetOptionsLoadEvent::EVENT_NAME)) {
            return $widgetOptions;
        }

        $event = new WidgetOptionsLoadEvent($widgetOptions, $type);
        $eventDispatcher->dispatch(WidgetOptionsLoadEvent::EVENT_NAME, $event);

        return $event->getWidgetOptions();
    }

    /**
     * @param array $options
     *
     * @return array
     */
    public function updateSegmentConditionBuilderOptions(array $options)
    {
        $eventDispatcher = $this->container->get('event_dispatcher');

        if (!$eventDispatcher->hasListeners(ConditionBuilderOptionsLoadEvent::EVENT_NAME)) {
            return $options;
        }

        $event = new ConditionBuilderOptionsLoadEvent($options);
        $eventDispatcher->dispatch(ConditionBuilderOptionsLoadEvent::EVENT_NAME, $event);

        return $event->getOptions();
    }

    /**
     * {@inheritdoc}
     */
    public function getName()
    {
        return static::NAME;
    }

    /**
     * {@inheritdoc}
     */
    public static function getSubscribedServices()
    {
        return [
            'event_dispatcher' => EventDispatcherInterface::class,
        ];
    }
}
